@@ -12,6 +12,9 @@
 #' If \code{svd.method = "rsvd"}, a randomized SVD is used,
 #' else if \code{svd.method = "tsvd"}, standard truncated SVD is used.
 #' Any unambiguous substring can be given. Default \code{svd.method = "tsvd"}.
+#' @param noise.var the variance of noise.
+#' @param init if init = FALSE(the default), the missing entries will initialize with mean.
+#' @param init.mat the initialization matrix.
 #' @param thresh convergence threshold, measured as the relative change in the Frobenius norm between two successive estimates.
 #' @param maxit	 maximal number of iterations.
 #' @param override logical value indicating whether the observed elements in \code{x} should be overwritten by its low-rank approximation.
@@ -41,8 +44,8 @@
 #' x_impute <- eimpute(x_na, r)
 #' head(x_impute[["x.imp"]][, 1:6])
 #' x_impute[["rmse"]]
-eimpute <- function(x, r, svd.method = c("tsvd", "rsvd"),
-                    thresh = 1e-05, maxit = 100, override = FALSE, control = list(...), ...){
+eimpute <- function(x, r, svd.method = c("tsvd", "rsvd"), noise.var = 0,
+                    thresh = 1e-05, maxit = 100, init = FALSE, init.mat = 0, override = FALSE, control = list(...), ...){
   m <- nrow(x)
   n <- ncol(x)
 
@@ -57,8 +60,18 @@ eimpute <- function(x, r, svd.method = c("tsvd", "rsvd"),
   svdm <- match.arg(svd.method)
   type <- ifelse(svdm == "tsvd", 1, 2)
 
+  ind_miss <- which(is.na(x_train), arr.ind = TRUE)
 
-  Z_temp <- kkt_fix(ind, x_ob, m, n, r, maxit, thresh, type)
+
+
+  if(init == TRUE){
+    ind_miss <- cbind(ind_miss, as.vector(init.mat[ind_miss]))
+  }else{
+    ind_miss <- cbind(ind_miss, rnorm(nrow(ind_miss), 0, noise.var))
+  }
+  ind_miss[,1:2] <- ind_miss[,1:2] - 1
+
+  Z_temp <- kkt_fix(ind, ind_miss, x_ob, m, n, r, maxit, thresh, type, init)
   Z.fit <- Z_temp[[1]] * (x_sd[[4]] %*% t(x_sd[[5]])) + matrix(rep(x_sd[[2]], n), nrow = m) + t(matrix(rep(x_sd[[3]], m), nrow = n))
 
 
